# python imports
from random import random

# zope imports
from DateTime import DateTime
from zope.interface import implements
from zope.component import adapts
from zope.interface import Interface
from zope.app.annotation.interfaces import IAnnotations
from BTrees.OOBTree import OOBTree
from DateTime import DateTime

# CMFCore imports
from Products.CMFCore.utils import getToolByName

# iqpp.commenting imports
from iqpp.commenting.interfaces import IComment
from iqpp.commenting.interfaces import ICommenting
from iqpp.commenting.interfaces import ICommentingOptions
from iqpp.commenting.interfaces import ICommentable
from iqpp.commenting.interfaces import ICommentTransformations
from iqpp.commenting.content import Comment
from iqpp.commenting.config import OPTIONS

KEY = "iqpp.commenting"

class Commenting:
    """An adapter, which provides commenting for commentable objects.
    """
    implements(ICommenting)
    adapts(ICommentable)

    def __init__(self, context):
        self.context = context
        annotations = IAnnotations(context)
        comments = annotations.get(KEY)
        
        if comments is None:
            comments = annotations[KEY] = OOBTree()

        self.comments = comments
        
    def addComment(self, reply_to, subject, message, member_id="", name="", email=""):
        """
        """
        new_id = self._generateUniqueId()
            
        options = ICommentingOptions(self.context).getOptions()
        if options.get("is_moderated"):
            review_state = "pending"
        else:
            review_state = "published"

        # create comment    
        self.comments[new_id] =\
            Comment(new_id, reply_to, subject, message, member_id, name, 
                    email, review_state)
                                    
        # transform message
        transformer = ICommentTransformations(self.comments[new_id])
        transformer.transformMessage()
        
        self.context.reindexObject()
        
        return new_id
        
    def deleteComment(self, id):
        """
        """        
        try:
            del self.comments[id]
            self.context.reindexObject()            
        except KeyError:
            return False
        else:
            for comment in self.comments.values():
                if comment.reply_to == id:
                    self.deleteComment(comment.id)
            return True
            
    def deleteComments(self):
        """
        """
        self.comments.clear()
        self.context.reindexObject()    
        
    def editComment(self, comment_id, reply_to, subject, message, member_id, name, email, review_state):
        """
        """
        comment = self.getComment(comment_id)        

        options = ICommentingOptions(self.context).getOptions()
        mtool = getToolByName(self.context, "portal_membership")
        if mtool.checkPermission("Manage portal", self.context) is None:
            if options.get("time_to_edit") < (DateTime() - comment.created*24*60):
                raise Unauthorized
                
        # User needs "Edit comments" in any case.
        if mtool.checkPermission("Edit comments", self.context) is None:
            raise Unauthorized
        
        comment.reply_to = reply_to
        comment.subject = subject
        comment.message = message
        comment.member_id = member_id
        comment.name = name
        comment.email = email
        comment.review_state = review_state
        comment.modified = DateTime()

        # transform message
        transformer = ICommentTransformations(comment)
        transformer.transformMessage()
        
        self.context.reindexObject()
        return True

    def getComment(self, id):
        """
        """
        try:
            return self.comments[id]
        except KeyError:
            return None

    def getAllComments(self, id=None):
        """
        """
        # Todo: implement for given id
        if id is None:
            return self.comments.values()
        else:
            raise "Not implemented yet."

    def getPendingComments(self):
        """
        """
        catalog = getToolByName(self.context, "portal_catalog")
        brains = catalog.searchResults(
            commentable = True
        )
        
        result = []
        for brain in brains:
            # catch "missing.value" in brain.comments
            if isinstance(getattr(brain, "comments", None), list):
                for comment_id in brain.comments:
                    result.append({
                        "object_url" : brain.getURL(),
                        "comment_id" : comment_id,
                    })
            
        result.sort(lambda a, b: cmp(b.created, a.created))        
        return result
        
    def getComments(self, id=""):
        """
        """        
        result = []
        for comment in self.comments.values():
            if comment.reply_to == id:
               result.append(comment)

        result.sort(lambda a, b: cmp(b.created, a.created))        
        return result
    
    def getCommentsForMember(self, member_id=None):
        """
        """    
        if member_id is None:
            mtool = getToolByName(self.context, "portal_membership")
            member_id = mtool.getAuthenticatedMember().getId()
            
        result = []
        for comment in self.comments.values():
            if comment.member_id == member_id:
               result.append(comment)

        result.sort(lambda a, b: cmp(b.created, a.created))
        return result
        
    def _generateUniqueId(self):
        """Generates a unique id
        """
        now  = DateTime()
        time = '%s.%s' % (now.strftime('%Y-%m-%d'), str(now.millis())[7:])
        rand = str(random())[2:6]

        return "comment."+time+rand

    def updateComment(self, comment_id, subject, message):
        """
        """
        comment = self.getComment(comment_id)

        comment.subject = subject
        comment.message = message

        # transform message
        transformer = ICommentTransformations(comment)
        transformer.transformMessage()
        
        self.context.reindexObject()
        return True
                
class CommentingOptions:
    """An adapter, which provides commenting options for commentable objects.
    """
    implements(ICommentingOptions)
    adapts(ICommentable)

    def __init__(self, context):
        self.context = context
        
    def getOptions(self):
        """
        """
        return OPTIONS
        
class CommentTransformations:
    """An adapter, which provides text transformations for comments.
    """
    implements(ICommentTransformations)
    adapts(IComment)

    def __init__(self, context):
        self.context = context
    
    def transformMessage(self):
        """
        """
        try:
            from intelligenttext.transforms import convertWebIntelligentPlainTextToHtml as convert
        except ImportError:
            transformed_message = self.context.message.replace("\n", "<br/>")
        else:    
            transformed_message = convert(self.context.message)
        
        self.context.transformed_message = transformed_message
                