# Zope imports
from DateTime import DateTime

# zope imports
from zope.interface import Interface
from zope.interface import implements
from zope.component import getMultiAdapter

from zope.i18nmessageid import MessageFactory
_ = MessageFactory("iqpp.commenting")

# Five imports
from Products.Five.browser import BrowserView
from Products.Five.browser import pagetemplatefile

# CMFCore imports
from Products.CMFCore.utils import getToolByName

# iqpp.commenting imports
from iqpp.commenting.config import MESSAGES, REVIEW_STATES
from iqpp.commenting.interfaces import ICommenting
from iqpp.commenting.interfaces import ICommentingOptions
from iqpp.commenting.interfaces import ICommentTransformations

from iqpp.commenting.browser.interfaces import IAddCommentView
from iqpp.commenting.browser.interfaces import IEditCommentView
from iqpp.commenting.browser.interfaces import ICommentsView

from iqpp.commenting.content import Comment

class AddCommentView(BrowserView):
    """
    """
    implements(IAddCommentView)
    template = pagetemplatefile.ZopeTwoPageTemplateFile("templates/add_comment_form.pt")    
   
    def __init__(self, context, request):
        """
        """
        super(AddCommentView, self).__init__(context, request)
        self._errors = {}

    def addComment(self, add_message=True, redirect=True, validate=True):
        """
        """
        utils = getToolByName(self.context, "plone_utils")
        mtool = getToolByName(self.context, "portal_membership")

        if validate == True:
            self.validate()
            if len(self._errors) > 0:
                utils.addPortalMessage(_("Please correct the indicated errors."))  
                # No redirect here, to make error handling work
                return self.template()
        
        if self.getOptions().get("show_preview") and \
           self.request.get("is_previewed", False) == False:
            preview = getMultiAdapter((self.context, self.request), name="preview_comment")
            return preview()

        subject    = self.request.get("subject", "")
        message    = self.request.get("message", "")
        comment_id = self.request.get("comment_id", "")
    
        if self.isMember() == False:
            member_id = None                        
        else:
            name = ""
            email = ""
            member_id = mtool.getAuthenticatedMember().getId()

        manager = ICommenting(self.context)
        if self.request.get("action") == "edit":
            manager.updateComment(
                comment_id = comment_id,
                subject = subject, 
                message = message,
            )
            if add_message == True:
                utils.addPortalMessage(_(MESSAGES["comment-modified"])) 
        else:            
            manager.addComment(
                reply_to = comment_id, 
                subject = subject, 
                message = message,
                name = name,
                email = email,
                member_id = member_id,
            )
            if add_message == True:            
                utils.addPortalMessage(_(MESSAGES["comment-added"]))             
        
        # redirect
        if redirect == True:
            self.redirect()
        
    def getFieldValues(self):
        """
        """    
        c = ICommenting(self.context)
        comment_id = self.request.get("comment_id")
        
        if self.request.get("action").lower() == "edit":
            comment = c.getComment(comment_id)
            name    = self.request.get("name")    or comment.name
            email   = self.request.get("email")   or comment.email
            subject = self.request.get("subject") or comment.subject
            message = self.request.get("message") or comment.message
            
        else:
            name    = self.request.get("name")
            email   = self.request.get("email")
            subject = self.request.get("subject")
            message = self.request.get("message")
            
        return {
            "id"           : comment_id,
            "name"         : name,
            "email"        : email,
            "subject"      : subject,
            "message"      : message,
        }
                            
    def getErrors(self):
        """
        """
        return self._errors

    def getOptions(self):
        """
        """    
        options = ICommentingOptions(self.context)
        return options.getOptions()

    def isMember(self):
        """
        """
        mtool = getToolByName(self.context, "portal_membership")
        return not mtool.isAnonymousUser()

    def redirect(self):
        """
        """
        url = self.context.absolute_url()            
        self.request.response.redirect(url)
    
    def validate(self):
        """
        """
        name    = self.request.get("name", "")
        email   = self.request.get("email", "")
        subject = self.request.get("subject", "")
        message = self.request.get("message", "")
        
        if subject == "":
            self._errors["subject"] = _("Subject is required.")
            
        if message == "":    
            self._errors["message"] = _("Message is required.")

        if self.isMember() == False:                        
            if name == "":
                self._errors["name"] = _("Name is required.")
            if email == "":
                self._errors["email"] = _("E-mail is required.")
        
class EditCommentView(BrowserView):
    """
    """
    implements(IEditCommentView)

    def editComment(self):
        """
        """
        utils = getToolByName(self.context, "plone_utils")
                    
        comment_id   = self.request.get("comment_id")
        reply_to     = self.request.get("reply_to")
        name         = self.request.get("name")
        email        = self.request.get("email")
        subject      = self.request.get("subject")
        message      = self.request.get("message")
        member_id    = self.request.get("member_id")
        review_state = self.request.get("review_state")
    
        manager = ICommenting(self.context)    
        result = manager.editComment(
            comment_id = comment_id,
            reply_to = reply_to, 
            name = name,
            email = email,
            subject = subject,
            message = message,
            member_id = member_id,
            review_state = review_state,
        )

        if result == True:
            utils.addPortalMessage(_(MESSAGES["comment-modified"]))
        else:
            utils.addPortalMessage(_(MESSAGES["comment-not-modified"]))
            
        url = self.context.absolute_url()
        self.request.response.redirect(url)
    
    def getComment(self):
        """
        """
        comment_id = self.request.get("comment_id")
        
        manager = ICommenting(self.context)
        comment = manager.getComment(comment_id)
        
        return {
            "id"           : comment.id,
            "name"         : comment.name,
            "email"        : comment.email,
            "subject"      : comment.subject,
            "message"      : comment.message,
            "reply_to"     : comment.reply_to,
            "member_id"    : comment.member_id,
            "review_state" : comment.review_state,
        }
        
    def getReviewStates(self):
        """
        """
        return REVIEW_STATES            

class CommentsView(BrowserView):
    """
    """
    def deleteComment(self, add_message=True, redirect=True):
        """
        """
        utils = getToolByName(self.context, "plone_utils")
        comment_id = self.request.get("comment_id")

        manager = ICommenting(self.context)
        result = manager.deleteComment(comment_id)

        if add_message == True:
            if result == True:
                utils.addPortalMessage(_(MESSAGES["comment-deleted"]))
            else:
                utils.addPortalMessage(_(MESSAGES["comment-not-deleted"]))

        if redirect == True:
            url = self.context.absolute_url()
            self.request.response.redirect(url)
        
    def deleteComments(self):
        """
        """    
        manager = ICommenting(self.context)
        manager.deleteComments()
        
        utils = getToolByName(self.context, "plone_utils")
        utils.addPortalMessage(_(MESSAGES["comments-deleted"]))
        
        url = self.context.absolute_url()
        self.request.response.redirect(url)

    def getComments(self, id=""):
        """
        """
        mtool = getToolByName(self.context, "portal_membership")
        
        if id is None:
            id = ""
        
        manager = ICommenting(self.context)
        comments = manager.getComments(id)
        
        result = []
        for i, comment in enumerate(comments):
            
            if mtool.checkPermission("Manage portal", self.context) is None and \
               comment.review_state != "published":
                continue
                   
            if i % 2 == 0:
                css_class = "mycomment even"
            else:
                css_class = "mycomment odd"
            
            member_info = _getMemberInfo(
                self.context, 
                comment.member_id, 
                comment.name, 
                comment.email
            )
            
            tool = getToolByName(self.context, 'translation_service')
            created_local = tool.ulocalized_time(comment.created, long_format=False)
            
            result.append({
                "id"                  : comment.id,
                "subject"             : comment.subject,
                "message"             : comment.message,
                "transformed_message" : comment.transformed_message,
                "name"                : member_info["name"],
                "email"               : member_info["email"],
                "css_class"           : css_class,
                "review_state"        : comment.review_state,
                "created"             : comment.created,
                "created_local"       : created_local,
                "member_id"           : comment.member_id,
            })
        
        return result

    def getCommentForMember(self, member_id=None):
        """
        """
        try:    
            return self.getCommentsForMember(member_id)[0]
        except IndexError:
            return []
        
    def getCommentsForMember(self, member_id=None):
        """
        """
        mtool = getToolByName(self.context, "portal_membership")
        cm = ICommenting(self.context)
        comments =  cm.getCommentsForMember(member_id)

        result = []
        for i, comment in enumerate(comments):
            
            if mtool.checkPermission("Manage portal", self.context) is None and \
               comment.review_state != "published":
                continue
                   
            if i % 2 == 0:
                css_class = "even"
            else:
                css_class = "odd"
            
            member_info = _getMemberInfo(
                self.context, 
                comment.member_id, 
                comment.name, 
                comment.email
            )
            
            result.append({
                "id"                  : comment.id,
                "subject"             : comment.subject,
                "message"             : comment.message,
                "transformed_message" : comment.transformed_message,
                "name"                : member_info["name"],
                "email"               : member_info["email"],
                "css_class"           : css_class,
                "review_state"        : comment.review_state,
                "created"             : comment.created,
                "member_id"           : comment.member_id,
            })
        
        return result

    def showDeleteButton(self, comment):
        """
        """

        # At the moment only manager can delete comments, with other words
        # owners are not allowed to delete their comments. This is supposed 
        # to be changed in future.
        
        mtool = getToolByName(self.context, "portal_membership")
        if mtool.checkPermission("Delete comments", self.context) == True:
            return True

        return False
                
    def showEditButton(self, comment):
        """
        """
        if self.showManageButton() == True:
            return False

        mtool = getToolByName(self.context, "portal_membership")        
        if comment["member_id"] == mtool.getAuthenticatedMember().getId():

            options = ICommentingOptions(self.context).getOptions()
            if options.get("time_to_edit") == -1:
                return True

            if options.get("time_to_edit") > ((DateTime() - comment["created"])*24*60):
                return True
                
        return False

    def showManageButton(self):
        """
        """
        # Permission
        mtool = getToolByName(self.context, "portal_membership")
        if mtool.checkPermission("Edit comments", self.context) == True:
            return True
        
        return False
        
class PreviewCommentView(BrowserView):
    """
    """    
    def getData(self):
        """
        """ 
        mtool = getToolByName(self.context, "portal_membership")
        
        if mtool.isAnonymousUser() == True:
            member_id = ""
            name  = self.request.get("name")
            email = self.request.get("email")
        else:
            member_id = mtool.getAuthenticatedMember().getId()
            name  = ""
            email = ""

        temp_comment = Comment(id="", reply_to="", subject="", message=self.request.get("message"))
        t = ICommentTransformations(temp_comment)
        t.transformMessage()
        
        member_info = _getMemberInfo(self.context, member_id, name, email)
                
        return {
            "name"                : member_info["name"],
            "email"               : member_info["email"],
            "subject"             : self.request.get("subject"),
            "transformed_message" : temp_comment.transformed_message,
            "message"             : temp_comment.message,
        }

class ReviewCommentsView(BrowserView):
    """
    """
    def getPendingComments(self):
        """
        """
        catalog = getToolByName(self.context, "portal_catalog")
        brains = catalog.searchResults(
            commentable = True
        )
        result = []
        for brain in brains:
            # catch "missing.value" in brain.comments
            if isinstance(getattr(brain, "comments", None), list):
                for comment_id in brain.comments:
                    result.append({
                        "object_url" : brain.getURL(),
                        "comment_id" : comment_id,
                    })
            
        # Todo: Find reasonable possibility to sort comments by date
        # result.sort(lambda a, b: cmp(b.created, a.created))        
        return result

    def getPendingCommentsForPortlet(self):
        """
        """
        result = self.getPendingComments()
        return result[:5]    

# # Helpers        
def _getMemberInfo(context, member_id, name, email):
    """
    """
    if name != "":
        name = name
    elif member_id != "":
        mtool = getToolByName(context, "portal_membership")                                           
        mi = mtool.getMemberInfo(member_id)            
        name =  mi and mi['fullname'] or member_id
    else:
        # This should never happen
        name = "Anonymous"

    if email != "":
        email = email
    elif member_id != "":
        mtool = getToolByName(context, "portal_membership")
        member = mtool.getMemberById(member_id)
        email = member and member.getProperty("email", "") or ""
    else:
        # This should never happen
        email = ""

    return {"name" : name, "email" : email}