# Zope imports
from zope.interface import Interface
from zope.app.annotation.interfaces import IAttributeAnnotatable

class ICommentable(IAttributeAnnotatable):
    """A marker interface to mark an object as commentable.
    """

class IComment(Interface):
    """Represents a comment.
    """    

class ICommentTransformations(Interface):
    """
    """
    def transformMessage():
        """Transforms a comment's message to arbitrary format. For instance
        to save html.
        """
        
class ICommenting(Interface):
    """A interface which provides methods to manage comments for arbitrary 
    objects.
    """

    def addComment(reply_to, subject, message, member_id=None, name=None, email=None):
        """Adds a comment.
        """
        
    def deleteComment(id):
        """Deletes a comment by given unique id and its replies.
        """

    def deleteComments():
        """Deletes all comments of an object.
        """

    def editComment(id, reply_to, subject, message, member_id, name, email):
        """Edits a existing comment.
        """

    def getAllComments(id=None):
        """Returns all comments from object with given id. If id is None,
        returns all comments of the root object.
        """

    def getComment(id):
        """Returns a comment by given uniqe id.
        """

    def getComments(id=None):
        """Returns direct comments of object with given id. If id is None, 
        returns comments of root object.
        """

    def getCommentsForMember(member_id):
        """Returns all comments for member with given member_id.
        """
        
    def updateComment(comment_id, subject, message):
        """Updates the subject and message of an existing comment.
        """
        
class ICommentingOptions(Interface):
    """Provides methods to manage options for commenting.
    """
    def getOptions():
        """Returns all options as a dict.
        """