# Zope imports
from AccessControl import Unauthorized

# zope imports
from zope.interface import directlyProvides
from zope.component import provideUtility, queryUtility

# test imports
from base import CommentingTestCase

# iqpp.rating imports
from iqpp.commenting.interfaces import ICommenting
from iqpp.commenting.interfaces import ICommentable

class TestCommentManagement(CommentingTestCase):

    def afterSetUp(self):
        self.setRoles(("Manager",))        
        
        self.portal.invokeFactory("Document", "document")
        self.d = self.portal.document
        
        # make the document commentable
        directlyProvides(self.d, ICommentable)
        
    def testAddComment(self):
        manager = ICommenting(self.d)
        new_id = manager.addComment(
                    reply_to = "", 
                    subject = "My Subject", 
                    message = "My Message", 
                    name="John Doe")

        comments = manager.getAllComments()
        self.assertEqual(len(comments), 1)
            
        comments = manager.getComments()
        self.assertEqual(len(comments), 1)
        
        comment = comments[0]
        self.assertEqual(comment.id, new_id)        
        self.assertEqual(comment.reply_to, "")
        self.assertEqual(comment.subject, "My Subject")
        self.assertEqual(comment.message, "My Message")
        self.assertEqual(comment.name, "John Doe")

        comments = manager.getComments(new_id)
        self.assertEqual(len(comments), 0)

        comment = manager.getComment(new_id)
        self.assertEqual(comment.id, new_id)        
        self.assertEqual(comment.reply_to, "")
        self.assertEqual(comment.subject, "My Subject")
        self.assertEqual(comment.message, "My Message")
        self.assertEqual(comment.name, "John Doe")

    def testAddCommentOfComment(self):
        manager = ICommenting(self.d)
        comment_id = manager.addComment(
                      reply_to = "", 
                      subject = "My Subject", 
                      message = "My Message", 
                      name="John Doe")
            
        reply_id = manager.addComment(
                      reply_to = comment_id, 
                      subject = "My Reply", 
                      message = "My Reply Message", 
                      name="Jane Doe")

        comments = manager.getAllComments()
        self.assertEqual(len(comments), 2)

        comments = manager.getComments()
        self.assertEqual(len(comments), 1)
        
        comments = manager.getComments(comment_id)
        self.assertEqual(len(comments), 1)

        comments = manager.getComments(reply_id)
        self.assertEqual(len(comments), 0)
        

    def testDeleteComment1(self):
        """
        """
        manager = ICommenting(self.d)
        comment_id_1 = manager.addComment(
                        reply_to = "", 
                        subject = "My Comment 1", 
                        message = "My Message 1", 
                        name="John Doe")

        reply_id = manager.addComment(
                        reply_to = comment_id_1, 
                        subject = "My Reply", 
                        message = "My Reply Message", 
                        name="Jane Doe")
            
        comment_id_2 = manager.addComment(
                        reply_to = "", 
                        subject = "My Comment 2", 
                        message = "My Message 2", 
                        name="John Doe")
                        
        comments = manager.getAllComments()
        self.assertEqual(len(comments), 3)

        # reply should also be deleted.
        manager.deleteComment(comment_id_1)        
        comments = manager.getAllComments()
        self.assertEqual(len(comments), 1)

    def testDeleteReply(self):
        """
        """
        manager = ICommenting(self.d)
        comment_id_1 = manager.addComment(
                        reply_to = "", 
                        subject = "My Comment 1", 
                        message = "My Message 1", 
                        name="John Doe")

        reply_id = manager.addComment(
                        reply_to = comment_id_1, 
                        subject = "My Reply", 
                        message = "My Reply Message", 
                        name="Jane Doe")
            
        comment_id_2 = manager.addComment(
                        reply_to = "", 
                        subject = "My Comment 2", 
                        message = "My Message 2", 
                        name="John Doe")
                        
        comments = manager.getAllComments()
        self.assertEqual(len(comments), 3)

        # reply should also be deleted.
        manager.deleteComment(reply_id)        
        comments = manager.getAllComments()
        self.assertEqual(len(comments), 2)

    def testDeleteComments(self):
        """
        """
        manager = ICommenting(self.d)
        comment_id_1 = manager.addComment(
                        reply_to = "", 
                        subject = "My Comment 1", 
                        message = "My Message 1", 
                        name="John Doe")

        reply_id = manager.addComment(
                        reply_to = comment_id_1, 
                        subject = "My Reply", 
                        message = "My Reply Message", 
                        name="Jane Doe")
            
        comment_id_2 = manager.addComment(
                        reply_to = "", 
                        subject = "My Comment 2", 
                        message = "My Message 2", 
                        name="John Doe")
                        
        # reply should also be deleted.
        manager.deleteComments()        
        comments = manager.getAllComments()
        self.assertEqual(len(comments), 0)
        
    def testEditComment(self):
        """
        """
        manager = ICommenting(self.d)
        comment_id_1 = manager.addComment(
                        reply_to = "", 
                        subject = "My Comment 1", 
                        message = "My Message 1", 
                        name="John Doe",)

        manager.editComment(
            comment_id = comment_id_1,
            reply_to = "",
            subject  = "Modified Subject",
            message  = "Modified Message",
            name     = "Jane Doe",
            member_id = "",
            email = "",
            review_state="public")
            
        comment = manager.getComment(comment_id_1)

        self.assertEqual(comment.id, comment_id_1)        
        self.assertEqual(comment.reply_to, "")
        self.assertEqual(comment.subject, "Modified Subject")
        self.assertEqual(comment.message, "Modified Message")
        self.assertEqual(comment.name, "Jane Doe")
        self.assertEqual(comment.review_state, "public")

                                
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestCommentManagement))        
    return suite
